@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,8 @@
 #include "storage/lru_cache.h"
 #include "usage_stats/usage_stats.h"
 
+using mozc::commands::Request;
+using mozc::dictionary::SuppressionDictionary;
 
 // This flag is set by predictor.cc
 // We can remove this after the ambiguity expansion feature get stable.
@@ -65,9 +67,6 @@ DEFINE_bool(enable_expansion_for_user_history_predictor,
             "enable ambiguity expansion for user_history_predictor.");
 
 namespace mozc {
-
-using commands::Request;
-
 namespace {
 // find suggestion candidates from the most recent 3000 history in LRU.
 // We don't check all history, since suggestion is called every key event
@@ -132,6 +131,19 @@ bool IsPunctuation(const string &value) {
           value == "\xEF\xBC\x8C" || value == "\xEF\xBC\x8E");
 }
 
+bool IsSentenceLikeCandidate(const Segment::Candidate &candidate) {
+  // A sentence should have a long reading.  Length check is done using key to
+  // absorb length difference in value variation, e.g.,
+  // "〜ください" and "〜下さい".
+  if (candidate.value.empty() || Util::CharsLen(candidate.key) < 8) {
+    return false;
+  }
+  // Our primary target sentence ends with Hiragana, e.g., "〜ます".
+  const ConstChar32ReverseIterator iter(candidate.value);
+  bool ret = Util::GetScriptType(iter.Get()) == Util::HIRAGANA;
+  return ret;
+}
+
 // Return romanaized string.
 string ToRoman(const string &str) {
   string result;
@@ -379,11 +391,13 @@ class UserHistoryPredictorSyncer : public Thread {
 UserHistoryPredictor::UserHistoryPredictor(
     const DictionaryInterface *dictionary,
     const POSMatcher *pos_matcher,
-    const SuppressionDictionary *suppression_dictionary)
+    const SuppressionDictionary *suppression_dictionary,
+    bool enable_content_word_learning)
     : dictionary_(dictionary),
       pos_matcher_(pos_matcher),
       suppression_dictionary_(suppression_dictionary),
       predictor_name_("UserHistoryPredictor"),
+      content_word_learning_enabled_(enable_content_word_learning),
       updated_(false),
       dic_(new DicCache(UserHistoryPredictor::cache_size())) {
   AsyncLoad();  // non-blocking
@@ -1634,7 +1648,9 @@ void UserHistoryPredictor::Finish(Segments *segments) {
       // Check if the previous value looks like a sentence.
       segments->history_segments_size() > 0 &&
       segments->history_segment(
-          segments->history_segments_size() - 1).candidates_size() > 0) {
+          segments->history_segments_size() - 1).candidates_size() > 0 &&
+      IsSentenceLikeCandidate(segments->history_segment(
+          segments->history_segments_size() - 1).candidate(0))) {
     const Entry *entry = &(dic_->Head()->value);
     DCHECK(entry);
     const string &last_value =
@@ -1695,6 +1711,8 @@ void UserHistoryPredictor::MakeLearningSegments(
     DCHECK_LE(1, segment.candidates_size());
     learning_segment.key = segment.candidate(0).key;
     learning_segment.value = segment.candidate(0).value;
+    learning_segment.content_key = segment.candidate(0).content_key;
+    learning_segment.content_value = segment.candidate(0).content_value;
     learning_segment.description = GetDescription(segment.candidate(0));
     learning_segments->push_back_history_segment(learning_segment);
   }
@@ -1705,24 +1723,20 @@ void UserHistoryPredictor::MakeLearningSegments(
       SegmentForLearning learning_segment;
       learning_segment.key = candidate.key;
       learning_segment.value = candidate.value;
+      learning_segment.content_key = candidate.content_key;
+      learning_segment.content_value = candidate.content_value;
       learning_segment.description = GetDescription(candidate);
       learning_segments->push_back_conversion_segment(learning_segment);
     } else {
-      int key_start_pos = 0, value_start_pos = 0;
-      for (size_t j = 0; j < candidate.inner_segment_boundary.size(); ++j) {
-        const int key_len = candidate.inner_segment_boundary[j].first;
-        const int value_len = candidate.inner_segment_boundary[j].second;
-        SegmentForLearning learning_segment;
-        Util::SubString(candidate.key, key_start_pos,
-                        key_len, &learning_segment.key);
-        Util::SubString(candidate.value, value_start_pos,
-                        value_len, &learning_segment.value);
+      SegmentForLearning learning_segment;
+      for (Segment::Candidate::InnerSegmentIterator iter(&candidate);
+           !iter.Done(); iter.Next()) {
+        iter.GetKey().CopyToString(&learning_segment.key);
+        iter.GetValue().CopyToString(&learning_segment.value);
+        iter.GetContentKey().CopyToString(&learning_segment.content_key);
+        iter.GetContentValue().CopyToString(&learning_segment.content_value);
         learning_segments->push_back_conversion_segment(learning_segment);
-        key_start_pos += key_len;
-        value_start_pos += value_len;
       }
-      DCHECK_EQ(key_start_pos, Util::CharsLen(candidate.key));
-      DCHECK_EQ(value_start_pos, Util::CharsLen(candidate.value));
     }
   }
 }
@@ -1770,6 +1784,15 @@ void UserHistoryPredictor::InsertHistory(bool is_suggestion_selected,
            segment.description,
            is_suggestion_selected, next_fp_to_set,
            last_access_time, segments);
+    if (content_word_learning_enabled_ &&
+        segment.content_key != segment.key &&
+        segment.content_value != segment.value) {
+      Insert(segment.content_key,
+             segment.content_value,
+             segment.description,
+             is_suggestion_selected, 0,
+             last_access_time, segments);
+    }
   }
 
   // Insert all_key/all_value
@@ -1789,6 +1812,12 @@ void UserHistoryPredictor::InsertHistory(bool is_suggestion_selected,
             segments->history_segments_size() - 1);
     const SegmentForLearning &conversion_segment =
         learning_segments.conversion_segment(0);
+    // Don't learn a link from/to a punctuation.  Note that another piece of
+    // code handles learning for (sentence + punctuation) form; see Finish().
+    if (IsPunctuation(history_segment.value) ||
+        IsPunctuation(conversion_segment.value)) {
+      return;
+    }
     Entry *history_entry = dic_->MutableLookupWithoutInsert(
         LearningSegmentFingerprint(history_segment));
     NextEntry next_entry;
